@@ -15,7 +15,7 @@ import re
 from flask import Flask, jsonify, request
 import socket
 import uuid
-from keras.models import load_model
+import tensorflow as tf 
 import base64
 import cv2
 from PIL import Image 
@@ -26,7 +26,6 @@ import shap
 # sys.path.append('.')
 import mlos.mloslogs as mlgs
 import mlos.invprocess as imvps
-import eli5
 import texthero as hero
 mloslgs = mlgs.logs()
 invpsx = imvps.invproc()
@@ -34,20 +33,38 @@ invpsx = imvps.invproc()
 class mlosmodel:
     def __init__(self):
         print("mlOS Deploy Model - Session Initialized...")
-    def getmodelinfo (self):
+    def getmodelinfo (self,deploytype=None):
         ismodelokay=False
+        volumedata='volumedata'
         _fn='deployedmodel.json'
+       
+
+        if(deploytype!=None):
+            if(deploytype=="challenger"):
+                volumedata='challengerdata'
+                _fn='challengermodel.json'
+        if(os.path.exists(_fn)==False):
+            return None
+        if(os.path.exists(volumedata)==False):
+            return None
         with open(_fn, 'r') as inpfile:  
             data=json.load(inpfile)
+
+        
         model_key = data["key"]
         model_ext =  data["ext"]
         model_base=  data["id"]
         model_gener=data["gener"]
         formattestdata= data["formattestdata"]
+
         fnmodel="model-" + model_key + "." +model_ext
-        fpmodel=os.path.join("volumedata",fnmodel)
-        trans_dir= os.path.join("volumedata","dbs","trans")
-        dtypeinfofn= os.path.join("volumedata","dbs","msample.json")
+        if(model_ext=="_noext_"):
+            fnmodel="model-" + model_key  
+        
+
+        fpmodel=os.path.join(volumedata,fnmodel)
+        trans_dir= os.path.join(volumedata,"dbs","trans")
+        dtypeinfofn= os.path.join(volumedata,"dbs","msample.json")
         dtypex=[]
         if os.path.exists(dtypeinfofn)==True:
             with open(dtypeinfofn, 'r') as infile:  
@@ -58,7 +75,7 @@ class mlosmodel:
         
 
         inifn="mconfig-" + model_key + ".json" 
-        iniffn=os.path.join("volumedata",inifn)
+        iniffn=os.path.join(volumedata,inifn)
         mlalgo="UN"
         if os.path.exists(iniffn)==True:
             with open(iniffn, 'r') as infile:  
@@ -67,17 +84,17 @@ class mlosmodel:
         cls_lbl=[]
         
         inifn= "class_label.json"
-        clsfn=os.path.join("volumedata",inifn)
+        clsfn=os.path.join(volumedata,inifn)
         if(os.path.exists(clsfn)):
             with open(clsfn, 'r') as infile:  
                 cinfo=json.load(infile)
             cls_lbl =cinfo["class_label"]
 
-        transformation_dir=os.path.join("volumedata","dbs","trans")
-        logpath= os.path.join("volumedata","logs" )
-        logfile= os.path.join("volumedata","logs", 'log.txt' )
-        fn_sample =os.path.join("volumedata","msample-"+model_key+".json") 
-        realtimecall=os.path.join("volumedata", 'realtime' )
+        transformation_dir=os.path.join(volumedata,"dbs","trans")
+        logpath= os.path.join(volumedata,"logs" )
+        logfile= os.path.join(volumedata,"logs", 'log.txt' )
+        fn_sample =os.path.join(volumedata,"msample-"+model_key+".json") 
+        realtimecall=os.path.join(volumedata, 'realtime' )
         if os.path.exists(realtimecall)==False:
             os.mkdir(realtimecall)
         rtfullname =os.path.join(realtimecall,"realtimeprediction.csv")
@@ -88,11 +105,11 @@ class mlosmodel:
         if os.path.exists(rtfnjson)==False:
             with open(rtfnjson, 'w') as outfile:  
                 json.dump(data,outfile)
-        apistatpath=os.path.join("volumedata", 'apistat' )
+        apistatpath=os.path.join(volumedata, 'apistat' )
         if os.path.exists(apistatpath)==False:
             os.mkdir(apistatpath)
 
-        countfile=os.path.join("volumedata", 'apistat',"count.json" )
+        countfile=os.path.join(volumedata, 'apistat',"count.json" )
         data={
             "predict_e":0,
             "getinfo_e":0,
@@ -106,7 +123,7 @@ class mlosmodel:
                 json.dump(data,outfile)
 
 
-        fn_mresult =os.path.join("volumedata","mresult-"+model_key+".json") 
+        fn_mresult =os.path.join(volumedata,"mresult-"+model_key+".json") 
         feimp=[]
         if os.path.exists(fn_mresult)==True:
             with open(fn_mresult, 'r') as infile:  
@@ -117,12 +134,13 @@ class mlosmodel:
             os.mkdir(logpath)
                 
         dbset=[]
-        fndbset=os.path.join("volumedata","dbs","dbset.json")
+        fndbset=os.path.join(volumedata,"dbs","dbset.json")
         if os.path.exists(fndbset):
             with open(fndbset, 'r') as infile:
                 dbset = json.load(infile)
 
         modelinfo ={
+            "volumedata":volumedata,
             "model_key":model_key,
             "model_gener":model_gener,
             "model_ext":model_ext,
@@ -158,20 +176,30 @@ class mlosmodel:
             }
             inc=inc+1
             lst.append(tmpx)
+        # print(lst)
         return lst
 
 
     def explainmodel(self,model,expinfo,trn_data, smpl, felist, feimp, mtype):
         itype="static"
         try:
+            # print(expinfo)
+            # mtype=  expinfo["mtype"]
+            # if(mtype=="usl"):
+            #     return "__unsupervised__" , "__unsupervised__"
+            volumedata='volumedata'
+            if("expinfo" in expinfo):
+                volumedata=expinfo['volumedata']
+
             fenrm= preprocessing.minmax_scale(feimp, feature_range=(0, 1), axis=0, copy=True)
-            explainerfn=os.path.join("volumedata", "explainer.json" )
+            explainerfn=os.path.join(volumedata, "explainer.json" )
             if(os.path.exists(explainerfn)):
                 _dtype = expinfo["dbset"]["dtype"]
                 if(_dtype=="nlp" or _dtype=="vision"):
                     itype="dynamic"
                     ret= self.makelist([1], felist,[1])
                     return itype, ret
+                dbset= expinfo["dbset"]
                 mtype=  expinfo["mtype"]
                 algo= expinfo["algo"]
                 ktype= expinfo["ktype"]
@@ -184,15 +212,25 @@ class mlosmodel:
                 f = ""
                 try:
                     if(mtype=="clsify" and algo=="dt"):
-                        f = lambda x: model.predict_proba(x)[:,1]
+                        f = lambda x: model.predict_proba(x)[0][:,1]
+                    elif(mtype=="clsify" and algo=="tree"):
+                        if(len(dbset["tst"])==1):
+                            f = lambda x: model.predict_proba(x)[:,1]
+                        else:
+                            f = lambda x: model.predict_proba(x)[0][:,1]
                     elif(mtype=="clsify" and algo=="deeplcnn"):
-                        f = lambda x: model.predict_proba(x)[:,1]
+                        if(len(dbset["tst"])==1):
+                            f = lambda x: model.predict_proba(x)[:,1]
+                        else:
+                            f = lambda x: model.predict_proba(x)[0][:,1]
                     elif(mtype=="regr" and algo=="df"):
-                        f = lambda x: model.predict(x)
+                        if(len(dbset["tst"])==1):
+                            f = lambda x: model.predict(x) 
+                        else:
+                            f = lambda x: model.predict(x) 
                     elif(mtype=="clsify" and algo=="decision_function"):
-                        f = lambda x: model.decision_function(x)[:,1]
+                        f = lambda x: model.decision_function(x)[0][:,1]
                 except:
-                    raise
                     pass
                 med =np.array(expinfo["med"]).reshape((1,smpl.shape[1]))
                 print(med)
@@ -205,13 +243,16 @@ class mlosmodel:
                 # print(med)
                 explainer=""
                 if(mtype=="clsify" and algo=="dt"):
-                    explainer = shap.KernelExplainer(f, med, feature_names = felist)
+                    print(med)
+                    explainer = shap.KernelExplainer(f, med, feature_names = dbset["trn"])
+                elif(mtype=="clsify" and algo =="tree"):
+                    explainer = shap.KernelExplainer(f, med, feature_names = dbset["trn"])
                 elif(mtype=="clsify" and algo=="decision_function"):
-                    explainer = shap.KernelExplainer(f, med, feature_names = felist)
+                    explainer = shap.KernelExplainer(f, med, feature_names = dbset["trn"])
                 elif(mtype=="regr" and ktype =="tree"):
-                    explainer=shap.TreeExplainer(model)
+                    explainer=shap.KernelExplainer(f, med,feature_names = dbset["trn"])
                 elif(mtype=="regr" and ktype =="other"):
-                    explainer=shap.KernelExplainer(f, med)
+                    explainer=shap.KernelExplainer(f, med,feature_names = dbset["trn"])
                 elif(mtype=="clsify" and algo=="deeplcnn"):
                     explainer = shap.KernelExplainer(f, med, feature_names = felist)
                 try:
@@ -235,22 +276,30 @@ class mlosmodel:
                 return itype, ret
             # impx=eli5.explain_prediction_df(model,smpl.iloc[0,:], feature_names = felist).to_json(orient='table')
         except:
-            raise
             ret= self.makelist(feimp, felist,fenrm)
             return itype, ret
     def loadmodel(self,modelinfo):
         model=None
         is_model_loaded=False
+        if(modelinfo==None):
+            return is_model_loaded, model
+        volumedata="volumedata"
+        if ("volumedata" in  modelinfo):
+            volumedata=modelinfo["volumedata"]
         fpmodel=modelinfo["model_full_path"]
+        print(fpmodel)
+
         if os.path.exists(fpmodel):
             if (modelinfo["model_ext"]=="bin"): 
                 model= fasttext.load_model(fpmodel)
             elif (modelinfo["model_ext"]=="h5"):
-                model = load_model(fpmodel)
+                model = tf.keras.models.load_model(fpmodel)
+            elif (modelinfo["model_ext"]=="_noext_"):
+                model = tf.keras.models.load_model(fpmodel)
             else:
                 with open(fpmodel, 'rb') as infile:  
                     model = pickle.load(infile)
-            fndbset=os.path.join("volumedata","dbs","dbset.json")
+            fndbset=os.path.join(volumedata,"dbs","dbset.json")
             with open(fndbset, 'r') as infile:
                 dbset = json.load(infile)
             # gn_dbset_info_stat=os.path.join("volumedata","datastat.json")
@@ -272,14 +321,17 @@ class mlosmodel:
     def getexplainermodel(self,modelinfo):
         model=None
         is_model_loaded=False
-        fpmodel=os.path.join("volumedata", "explainer.pkl" )
-        exptrnfn=os.path.join("volumedata", "xptrn.pkl" )
+        volumedata="volumedata"
+        if("volumedata" in modelinfo):
+            volumedata=modelinfo["volumedata"]
+        fpmodel=os.path.join(volumedata, "explainer.pkl" )
+        exptrnfn=os.path.join(volumedata, "xptrn.pkl" )
         trn_data=None
         expinfo={}
         if(os.path.exists(exptrnfn)):
             with open(exptrnfn, 'rb') as infile:  
                 trn_data = pickle.load(infile)
-        explainerfn=os.path.join("volumedata", "explainer.json" )
+        explainerfn=os.path.join(volumedata, "explainer.json" )
         if(os.path.exists(explainerfn)):
             with open (explainerfn,'r') as infile:
                 expinfo= json.load(infile)
@@ -292,10 +344,11 @@ class mlosmodel:
                 is_model_loaded=False
                 model =None
         else:
-            explainerfn=os.path.join("volumedata", "explainer.json" )
+            explainerfn=os.path.join(volumedata, "explainer.json" )
             if(os.path.exists(explainerfn)):
                 with open (explainerfn,'r') as infile:
                     expinfo= json.load(infile)
+                    expinfo["volumedata"]=volumedata
             is_model_loaded=False
         return is_model_loaded,expinfo, model, trn_data
     def prepare_test_data(self,data,modelinfo):
@@ -490,8 +543,15 @@ class mlosmodel:
                 if (model_ext=="bin"):
                     arrx=X_test[dbset["trn"][0]].values
                     for a in arrx:
-                        art.append(a)
-                    y_pred_tmp = model.predict(art,k=1)
+                        art.append(a) 
+                    try:
+                        y_pred_tmp = model.predict(art,k=1)
+                    except:
+                        try:
+                            if(mtype=="usl"):
+                                y_pred_tmp = model.fit_predict(art)
+                        except:
+                            pass
                     if(mtype=="clsify"):
                         y_prob = model.predict_proba(art)
             except:
@@ -503,7 +563,7 @@ class mlosmodel:
                 result={'results':vt,"status":"OK","success":True,"error":False}
                 return result
 
-        elif (model_ext=="h5"):
+        elif (model_ext=="h5" or model_ext=="_noext_"):
             if formattestdata=="deepl1":
                 try:
                     xtst_shape=X_test.shape
@@ -534,12 +594,24 @@ class mlosmodel:
                     return result
         else:
             try:
-                y_pred_tmp = model.predict(X_test)
+                try:
+                    y_pred_tmp = model.predict(X_test)
+                except:
+                    if(mtype=="usl"):
+                        y_pred_tmp = model.fit_predict(X_test)    
+                    
+                    
                 if(mtype=="clsify"):
                     if(modelinfo["algo"]=="LinearSVC"):
-                        y_prob = model.decision_function(X_test)
+                        try:
+                            y_prob = model.decision_function(X_test)
+                        except:
+                            pass
                     else:
-                        y_prob = model.predict_proba(X_test)
+                        try:
+                            y_prob = model.predict_proba(X_test)
+                        except:
+                            pass
             except:
                 vt = {
                     "success":False,
@@ -554,12 +626,25 @@ class mlosmodel:
             y_pred= np.asarray(y_pred_tmp[0]).ravel()
         else:
             y_pred=y_pred_tmp
+        print("=====")
+        print(y_prob)
+        
+        print("=====")
+        tmp_prob=[]
+        if(len(y_prob)>1):
+            print("Multi Target")
+            
+            for v in y_prob:
+                tmp_prob.append( np.around(v,2).tolist())
+        else:
+            tmp_prob=np.around(y_prob,2)
+
 
         vt = {
             "success":True,
             "error":False,
             "prediction":y_pred,
-            "probability":np.array(np.around(y_prob,2) ) 
+            "probability": np.array(tmp_prob)
         }
         result={'results':vt,"status":"OK","success":True,"error":False}
         return result
@@ -567,45 +652,115 @@ class mlosmodel:
 
 
     def saverowcount(self,fn,ncount):
-        with open(fn, 'r') as inpfile:  
-            data=json.load(inpfile)
-        count_rows=data["rows"] + ncount
-        data={
-            "rows":count_rows
-        }
-        with open(fn, 'w') as outfile:  
-            json.dump(data,outfile)   
+        try:
+            with open(fn, 'r') as inpfile:  
+                data=json.load(inpfile)
+            count_rows=int(data["rows"])  + int(ncount)
+            data={
+                "rows":count_rows
+            }
+            with open(fn, 'w') as outfile:  
+                json.dump(data,outfile)   
+        except:
+            
+            pass
 
     def prepare_restored_response(self,y_pred,_saveX_test,modelinfo):
         try:
             colt=modelinfo["dbset"]["tst"]
+            print(colt)
             logfile=modelinfo ["logfile"]
             ts_fn=os.path.join(modelinfo["transformation_dir"],colt[0]+ "+categorical_2_numeric"+ ".pkl")
             mloslgs.logtext(logfile,ts_fn,False) 
             out_con=[]
             reconstrcted_response=[]
             dfx=[]
-        
-            try:
-                if os.path.exists(ts_fn)==True:
-                    out_con=invpsx.getinvtransfromed(ts_fn, y_pred)
-                    mloslgs.logtext(logfile,out_con,False)
-            except:
+            print("=========PRED===========")
+            print(y_pred)
+            ln=1
+            lnSmpl=len(y_pred)
+            for v in y_pred:
                 try:
-                    valx=[]
-                    for v in y_pred:
-                        valx.append(int(v))
-                    out_con=invpsx.getinvtransfromed(ts_fn, valx)
-                    mloslgs.logtext(logfile,"ecepet",False)
+                    ln=len(v)
                 except:
-                    mloslgs.logtext(logfile,"ERRROR",False)
-                    out_con=[]
+                    pass
+            print(ln)
+            _colnames=['predicted']
+            _colnames_trns=['predicted_transformed']
+            print("====================")
+            if(ln==1):
+                print("Single target")
+                try:
+                    if os.path.exists(ts_fn)==True:
 
+                        out_con=invpsx.getinvtransfromed(ts_fn, y_pred)
+                        mloslgs.logtext(logfile,out_con,False)
+                except:
+                    try:
+                        valx=[]
+                        for v in y_pred:
+                            valx.append(int(v))
+                        out_con=invpsx.getinvtransfromed(ts_fn, valx)
+                        mloslgs.logtext(logfile,"ecepet",False)
+                    except:
+                        print("ERR IN CON")
+                        raise
+                        mloslgs.logtext(logfile,"ERRROR",False)
+                        out_con=[]
+            else:
+                _colnames=[]
+                _colnames_trns=[]
+ 
+                print("Multi target")
+                out_con=[]
+                colt=modelinfo["dbset"]["tst"]
+                inc=0
+                tdx_ppx=[]
+                print(y_pred)
+                for cl in colt:
+                    _colnames.append("predicted_"+cl)
+                    _colnames_trns.append("predicted_trns_"+cl)
+                    y_pred_new=[]
+                    for px_prd in y_pred:
+                        y_pred_new.append(px_prd[inc])
+                    ts_fn=os.path.join(modelinfo["transformation_dir"],cl+ "+categorical_2_numeric"+ ".pkl")
+                    
+                    if os.path.exists(ts_fn)==True:
+                        y_pred_new=[]
+                        for px_prd in y_pred:
+                            y_pred_new.append(int(px_prd[inc]))
+                        try:
+                            print(y_pred_new)
+                            out_con=invpsx.getinvtransfromed(ts_fn, y_pred_new)
+                            print(out_con)
+                            mloslgs.logtext(logfile,out_con,False)
+                        except:
+                            print("ERROR INV ER")
+                            raise
+                            out_con=y_pred_new
+                        tdx_ppx.append(np.array(out_con))
+                    else:
+                        tdx_ppx.append(np.array(y_pred_new) )
+                    inc=inc+1
+                
+                print(tdx_ppx)
+                new_pred_conv=[]
+                for i in range(lnSmpl):
+                    _new_vx=[]
+                    for vx in tdx_ppx:
+                        _new_vx.append(vx[i])
+                    new_pred_conv.append(_new_vx)
+
+                print(new_pred_conv)
+                out_con=np.array(new_pred_conv)
+            print("=========CON===========")
+            print(out_con)
+            print("====================")
             if len(out_con)>0:
-                dfx=pd.concat([_saveX_test,pd.DataFrame(y_pred,columns=['predicted'] ),pd.DataFrame(out_con,columns=['predicted_transformed'] )],axis=1)
+                dfx=pd.concat([_saveX_test,pd.DataFrame(y_pred,columns=_colnames ),pd.DataFrame(out_con,columns=_colnames_trns )],axis=1)
                 reconstrcted_response=out_con.tolist()
             else:
-                dfx=pd.concat([_saveX_test,pd.DataFrame(y_pred,columns=['predicted'] ),pd.DataFrame(y_pred,columns=['predicted_transformed'] )],axis=1)
+                dfx=pd.concat([_saveX_test,pd.DataFrame(y_pred,columns=_colnames ),pd.DataFrame(y_pred,columns=_colnames_trns )],axis=1)
                 reconstrcted_response=y_pred.tolist()
 
             rows,cols=dfx.shape
@@ -614,8 +769,11 @@ class mlosmodel:
             if os.path.exists(realtime_data_file)==False:
                 with open(realtime_data_file, 'w') as outfile:
                     dfx.to_csv(outfile,index=False)
-                rows,cols=dfx.shape
-                self.saverowcount(realtime_row_count_file,rows)
+                try:
+                    rows,cols=dfx.shape
+                    self.saverowcount(realtime_row_count_file,rows)
+                except:
+                    pass
             else:
                 x = dfx.to_string(header=False,index=False,index_names=False).split('\n')
                 vals = [','.join(ele.split()) for ele in x]
@@ -625,10 +783,20 @@ class mlosmodel:
                         fd.write("\n"+vx)
                 mloslgs.logtext(logfile,vals,False)  
                 self.saverowcount(realtime_row_count_file,rows)
-            return  reconstrcted_response
+            retval= reconstrcted_response
+            try:
+                retval=reconstrcted_response.tolist()
+            except:
+                pass
+            return  retval
         except:
+            raise
             # dfx=pd.concat([_saveX_test,pd.DataFrame(y_pred,columns=['predicted'] ),pd.DataFrame(y_pred,columns=['predicted_transformed'] )],axis=1)
             reconstrcted_response=y_pred
+            try:
+                reconstrcted_response=y_pred.tolist()
+            except:
+                pass
             return reconstrcted_response
             
 
